from enum import Enum
from typing import TypeAlias

from google.genai import types

class GeminiRole(Enum):
    User = "user"
    Model = "model"

GeminiMessage: TypeAlias = types.Content

# class Content(_common.BaseModel):
#   """Contains the multi-part content of a message."""
#
#   parts: Optional[list[Part]] = Field(
#       default=None,
#       description="""List of parts that constitute a single message. Each part may have
#       a different IANA MIME type.""",
#   )
#   role: Optional[str] = Field(
#       default=None,
#       description="""Optional. The producer of the content. Must be either 'user' or
#       'model'. Useful to set for multi-turn conversations, otherwise can be
#       left blank or unset. If role is not specified, SDK will determine the role.""",
#   )
#
# class Part(_common.BaseModel):
#   """A datatype containing media content.
#
#   Exactly one field within a Part should be set, representing the specific type
#   of content being conveyed. Using multiple fields within the same `Part`
#   instance is considered invalid.
#   """
#
#   video_metadata: Optional[VideoMetadata] = Field(
#       default=None, description="""Metadata for a given video."""
#   )
#   thought: Optional[bool] = Field(
#       default=None,
#       description="""Indicates if the part is thought from the model.""",
#   )
#   code_execution_result: Optional[CodeExecutionResult] = Field(
#       default=None,
#       description="""Optional. Result of executing the [ExecutableCode].""",
#   )
#   executable_code: Optional[ExecutableCode] = Field(
#       default=None,
#       description="""Optional. Code generated by the model that is meant to be executed.""",
#   )
#   file_data: Optional[FileData] = Field(
#       default=None, description="""Optional. URI based data."""
#   )
#   function_call: Optional[FunctionCall] = Field(
#       default=None,
#       description="""Optional. A predicted [FunctionCall] returned from the model that contains a string representing the [FunctionDeclaration.name] with the parameters and their values.""",
#   )
#   function_response: Optional[FunctionResponse] = Field(
#       default=None,
#       description="""Optional. The result output of a [FunctionCall] that contains a string representing the [FunctionDeclaration.name] and a structured JSON object containing any output from the function call. It is used as context to the model.""",
#   )
#   inline_data: Optional[Blob] = Field(
#       default=None, description="""Optional. Inlined bytes data."""
#   )
#   text: Optional[str] = Field(
#       default=None, description="""Optional. Text part (can be code)."""
#   )
#
#   @classmethod
#   def from_uri(cls, file_uri: str, mime_type: str) -> 'Part':
#     file_data = FileData(file_uri=file_uri, mime_type=mime_type)
#     return cls(file_data=file_data)
#
#   @classmethod
#   def from_text(cls, text: str) -> 'Part':
#     return cls(text=text)
#
#   @classmethod
#   def from_bytes(cls, data: bytes, mime_type: str) -> 'Part':
#     inline_data = Blob(
#         data=data,
#         mime_type=mime_type,
#     )
#     return cls(inline_data=inline_data)
#
#   @classmethod
#   def from_function_call(cls, name: str, args: dict[str, Any]) -> 'Part':
#     function_call = FunctionCall(name=name, args=args)
#     return cls(function_call=function_call)
#
#   @classmethod
#   def from_function_response(
#       cls, name: str, response: dict[str, Any]
#   ) -> 'Part':
#     function_response = FunctionResponse(name=name, response=response)
#     return cls(function_response=function_response)
#
#   @classmethod
#   def from_video_metadata(cls, end_offset: str, start_offset: str) -> 'Part':
#     video_metadata = VideoMetadata(
#         end_offset=end_offset, start_offset=start_offset
#     )
#     return cls(video_metadata=video_metadata)
#
#   @classmethod
#   def from_executable_code(cls, code: str, language: Language) -> 'Part':
#     executable_code = ExecutableCode(code=code, language=language)
#     return cls(executable_code=executable_code)
#
#   @classmethod
#   def from_code_execution_result(cls, outcome: Outcome, output: str) -> 'Part':
#     code_execution_result = CodeExecutionResult(outcome=outcome, output=output)
#     return cls(code_execution_result=code_execution_result)



import mimetypes
from typing import Optional

from google.genai import types
from google.genai.types import Part

from llm_bridge.client.implementations.gemini.gemini_token_counter import count_gemini_tokens
from llm_bridge.client.implementations.printing_status import PrintingStatus
from llm_bridge.logic.chat_generate.media_processor import bytes_to_base64
from llm_bridge.type.chat_response import ChatResponse, File


class GeminiResponseHandler:
    def __init__(self):
        self.printing_status: Optional[PrintingStatus] = None
        self.prev_output_tokens: int = 0
        self.prev_printing_status: Optional[PrintingStatus] = None

    async def process_gemini_response(
            self,
            response: types.GenerateContentResponse,
    ) -> ChatResponse:
        text: str = ""
        thought: str = ""
        code: str = ""
        code_output: str = ""
        files: list[File] = []
        display: Optional[str] = None
        input_tokens, stage_output_tokens = await count_gemini_tokens(response)

        parts: list[Part] = []
        if candidates := response.candidates:
            if content := candidates[0].content:
                if content.parts:
                    parts = content.parts

        printing_status: PrintingStatus | None = None
        for part in parts:
            if part.text is not None:
                # Thought
                if part.thought:
                    printing_status = PrintingStatus.Thought
                    thought += part.text
                # Text
                elif not part.thought:
                    printing_status = PrintingStatus.Response
                    text += part.text
            # Code
            if part.executable_code is not None:
                code += part.executable_code.code
            # Code Output
            if part.code_execution_result is not None:
                code_output += part.code_execution_result.output
            # File
            if part.inline_data is not None:
                mime_type = part.inline_data.mime_type
                extension = mimetypes.guess_extension(mime_type) or ""
                file = File(
                    name=f"generated_file{extension}",
                    data=bytes_to_base64(part.inline_data.data),
                    type=mime_type,
                )
                files.append(file)

        # Grounding Sources
        if candidates := response.candidates:
            if grounding_metadata := candidates[0].grounding_metadata:
                if search_entry_point := grounding_metadata.search_entry_point:
                    display = search_entry_point.rendered_content
                if grounding_metadata.grounding_chunks:
                    text += "\n\n# Grounding Sources:\n"
                    for i, chunk in enumerate(grounding_metadata.grounding_chunks, start=1):
                        if chunk.web:
                            text += f"{i}. [{chunk.web.title}]({chunk.web.uri})\n"

        if printing_status == self.prev_printing_status and printing_status == PrintingStatus.Response:
            output_tokens = stage_output_tokens - self.prev_output_tokens
        else:
            output_tokens = stage_output_tokens

        self.prev_output_tokens = stage_output_tokens
        self.prev_printing_status = printing_status

        return ChatResponse(
            text=text,
            thought=thought,
            code=code,
            code_output=code_output,
            files=files,
            display=display,
            input_tokens=input_tokens,
            output_tokens=output_tokens,
        )



from anthropic import BetaMessageStreamEvent, AsyncAnthropic
from anthropic._response import AsyncBinaryAPIResponse
from anthropic.types.beta import BetaRawContentBlockDelta, BetaThinkingDelta, BetaTextDelta, BetaInputJSONDelta, \
    BetaBashCodeExecutionToolResultBlock, \
    BetaTextEditorCodeExecutionToolResultBlock, BetaTextEditorCodeExecutionViewResultBlock, \
    BetaTextEditorCodeExecutionStrReplaceResultBlock, \
    BetaServerToolUseBlock, BetaBashCodeExecutionResultBlock, BetaTextBlock, BetaThinkingBlock, \
    BetaBashCodeExecutionOutputBlock, BetaMessage, FileMetadata
from anthropic.types.beta.beta_raw_content_block_start_event import ContentBlock

from llm_bridge.client.implementations.claude.claude_token_counter import count_claude_output_tokens
from llm_bridge.logic.chat_generate.media_processor import bytes_to_base64
from llm_bridge.type.chat_response import ChatResponse, File


async def download_claude_file(client: AsyncAnthropic, file_id: str) -> File:
    file_metadata: FileMetadata = await client.beta.files.retrieve_metadata(file_id)
    file_content: AsyncBinaryAPIResponse = await client.beta.files.download(file_id)
    data = await file_content.read()
    return File(
        name=file_metadata.filename,
        data=bytes_to_base64(data),
        type=file_metadata.mime_type,
    )

async def process_content_block(
        content_block: ContentBlock, client: AsyncAnthropic
) -> ChatResponse:
    text: str = ""
    thought: str = ""
    code: str = ""
    code_output: str = ""
    files: list[File] = []

    if content_block.type == "text":
        text_block: BetaTextBlock = content_block
        text += text_block.text

    elif content_block.type == "thinking":
        thinking_block: BetaThinkingBlock = content_block
        thought += thinking_block.thinking

    elif content_block.type == "server_tool_use":
        server_tool_use_block: BetaServerToolUseBlock = content_block
        code += str(server_tool_use_block.input)

    elif content_block.type == "bash_code_execution_tool_result":
        bash_code_execution_tool_result_block: BetaBashCodeExecutionToolResultBlock = content_block
        if bash_code_execution_tool_result_block.content.type == "bash_code_execution_result":
            result: BetaBashCodeExecutionResultBlock = content_block.content
            code_output += result.stdout
            outputs: list[BetaBashCodeExecutionOutputBlock] = result.content
            file_ids = [output.file_id for output in outputs]
            for file_id in file_ids:
                file = await download_claude_file(client, file_id)
                files.append(file)

    elif content_block.type == "text_editor_code_execution_tool_result":
        text_editor_code_execution_tool_result: BetaTextEditorCodeExecutionToolResultBlock = content_block
        if text_editor_code_execution_tool_result.content.type == "text_editor_code_execution_view_result":
            result: BetaTextEditorCodeExecutionViewResultBlock = content_block.content
            code_output += result.content
        elif text_editor_code_execution_tool_result.content.type == "text_editor_code_execution_str_replace_result":
            result: BetaTextEditorCodeExecutionStrReplaceResultBlock = content_block.content
            code_output += result.lines

    return ChatResponse(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
    )


async def build_chat_response_with_tokens(
        text: str,
        thought: str,
        code: str,
        code_output: str,
        files: list[File],
        input_tokens: int,
        client: AsyncAnthropic,
        model: str,
) -> ChatResponse:
    chat_response = ChatResponse(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
    )
    output_tokens = await count_claude_output_tokens(
        client=client,
        model=model,
        chat_response=chat_response,
    )
    return ChatResponse(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
        input_tokens=input_tokens,
        output_tokens=output_tokens,
    )


async def process_claude_non_stream_response(
        message: BetaMessage,
        input_tokens: int,
        client: AsyncAnthropic,
        model: str,
) -> ChatResponse:
    text = ""
    thought = ""
    code = ""
    code_output = ""
    files: list[File] = []

    for content_block in message.content:
        content_block_chat_response = await process_content_block(content_block, client)
        text += content_block_chat_response.text
        thought += content_block_chat_response.thought
        code += content_block_chat_response.code
        code_output += content_block_chat_response.code_output
        files.extend(content_block_chat_response.files)

    return await build_chat_response_with_tokens(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
        input_tokens=input_tokens,
        client=client,
        model=model,
    )


async def process_claude_stream_response(
        event: BetaMessageStreamEvent,
        input_tokens: int,
        client: AsyncAnthropic,
        model: str,
) -> ChatResponse:
    text = ""
    thought = ""
    code = ""
    code_output = ""
    files: list[File] = []

    if event.type == "content_block_delta":
        event_delta: BetaRawContentBlockDelta = event.delta

        if event_delta.type == "text_delta":
            text_delta: BetaTextDelta = event_delta
            text += text_delta.text

        elif event_delta.type == "thinking_delta":
            thinking_delta: BetaThinkingDelta = event_delta
            thought += thinking_delta.thinking

        elif event_delta.type == "input_json_delta":
            input_json_delta: BetaInputJSONDelta = event_delta
            code += input_json_delta.partial_json

    if event.type == "content_block_start":
        content_block: ContentBlock = event.content_block
        content_block_chat_response = await process_content_block(content_block, client)
        text += content_block_chat_response.text
        thought += content_block_chat_response.thought
        code += content_block_chat_response.code
        code_output += content_block_chat_response.code_output
        files.extend(content_block_chat_response.files)

    return await build_chat_response_with_tokens(
        text=text,
        thought=thought,
        code=code,
        code_output=code_output,
        files=files,
        input_tokens=input_tokens,
        client=client,
        model=model,
    )



from typing import Any

import anthropic
from anthropic import Omit, transform_schema
from anthropic.types import ThinkingConfigEnabledParam, AnthropicBetaParam
from anthropic.types.beta import BetaWebSearchTool20250305Param, BetaToolUnionParam, BetaCodeExecutionTool20250825Param, \
    BetaToolBash20250124Param, BetaJSONOutputFormatParam

from llm_bridge.client.implementations.claude.claude_token_counter import count_claude_input_tokens
from llm_bridge.client.implementations.claude.non_stream_claude_client import NonStreamClaudeClient
from llm_bridge.client.implementations.claude.stream_claude_client import StreamClaudeClient
from llm_bridge.logic.chat_generate.chat_message_converter import convert_messages_to_claude
from llm_bridge.logic.message_preprocess.message_preprocessor import extract_system_messages
from llm_bridge.type.message import Message


async def create_claude_client(
        api_key: str,
        messages: list[Message],
        model: str,
        temperature: float,
        stream: bool,
        thought: bool,
        code_execution: bool,
        structured_output_schema: dict[str, Any] | None
):
    omit = Omit()

    client = anthropic.AsyncAnthropic(
        api_key=api_key,
    )

    system = await extract_system_messages(messages)
    if system == "":
        system = "/"

    claude_messages = await convert_messages_to_claude(messages)

    input_tokens = await count_claude_input_tokens(
        client=client,
        model=model,
        system=system,
        messages=claude_messages,
    )

    context_window = 200_000
    if model in ["claude-sonnet-4-5"]:
        context_window = 1_000_000
    max_output = 64_000
    max_tokens = min(
        max_output,
        context_window - input_tokens,
    )

    thinking: ThinkingConfigEnabledParam | Omit = omit
    if thought:
        thinking = ThinkingConfigEnabledParam(
            type="enabled",
            budget_tokens=max(1024, max_tokens // 2),  # Minimum budget tokens: 1024
        )
        temperature = 1

    betas: list[AnthropicBetaParam] = [
        "context-1m-2025-08-07",
        "output-128k-2025-02-19",
        "code-execution-2025-08-25",
        "files-api-2025-04-14",
        "structured-outputs-2025-11-13",
    ]

    tools: list[BetaToolUnionParam] = []
    tools.append(
        BetaWebSearchTool20250305Param(
            type="web_search_20250305",
            name="web_search",
        )
    )
    if code_execution:
        tools.append(
            BetaToolBash20250124Param(
                type="bash_20250124",
                name="bash",
            )
        )
        tools.append(
            BetaCodeExecutionTool20250825Param(
                type="code_execution_20250825",
                name="code_execution",
            )
        )

    extra_body = None
    if structured_output_schema:
        extra_body = {
            "output_format": BetaJSONOutputFormatParam(
                type="json_schema",
                schema=transform_schema(structured_output_schema),
            )
        }

    if stream:
        return StreamClaudeClient(
            model=model,
            messages=claude_messages,
            temperature=temperature,
            system=system,
            client=client,
            max_tokens=max_tokens,
            betas=betas,
            input_tokens=input_tokens,
            tools=tools,
            thinking=thinking,
            extra_body=extra_body,
        )
    else:
        return NonStreamClaudeClient(
            model=model,
            messages=claude_messages,
            temperature=temperature,
            system=system,
            client=client,
            max_tokens=max_tokens,
            betas=betas,
            input_tokens=input_tokens,
            tools=tools,
            thinking=thinking,
            extra_body=extra_body,
        )



import re
from typing import Any

import openai
from fastapi import HTTPException
from openai import Omit
from openai.types import Reasoning
from openai.types.responses import WebSearchToolParam, ResponseIncludable, ResponseTextConfigParam, \
    ResponseFormatTextJSONSchemaConfigParam
from openai.types.responses.tool_param import CodeInterpreter, CodeInterpreterContainerCodeInterpreterToolAuto, \
    ImageGeneration, ToolParam

from llm_bridge.client.implementations.openai.non_stream_openai_client import NonStreamOpenAIClient
from llm_bridge.client.implementations.openai.non_stream_openai_responses_client import NonStreamOpenAIResponsesClient
from llm_bridge.client.implementations.openai.steam_openai_responses_client import StreamOpenAIResponsesClient
from llm_bridge.client.implementations.openai.stream_openai_client import StreamOpenAIClient
from llm_bridge.logic.chat_generate.chat_message_converter import convert_messages_to_openai_responses, \
    convert_messages_to_openai
from llm_bridge.type.message import Message


async def create_openai_client(
        api_keys: dict[str, str],
        messages: list[Message],
        model: str,
        api_type: str,
        temperature: float,
        stream: bool,
        thought: bool,
        code_execution: bool,
        structured_output_schema: dict[str, Any] | None,
):
    omit = Omit()

    if api_type == "OpenAI":
        client = openai.AsyncOpenAI(
            api_key=api_keys["OPENAI_API_KEY"],
        )
    elif api_type == "OpenAI-Azure":
        client = openai.AsyncAzureOpenAI(
            api_version="2025-03-01-preview",
            azure_endpoint=api_keys["AZURE_API_BASE"],
            api_key=api_keys["AZURE_API_KEY"],
        )
    elif api_type == "OpenAI-GitHub":
        client = openai.AsyncOpenAI(
            base_url="https://models.inference.ai.azure.com",
            api_key=api_keys["GITHUB_API_KEY"],
        )
    elif api_type == "Grok":
        client = openai.AsyncOpenAI(
            base_url="https://api.x.ai/v1",
            api_key=api_keys["XAI_API_KEY"],
        )
    else:
        raise HTTPException(status_code=500, detail="API Type not matched")

    if api_type in ("OpenAI", "OpenAI-Azure"):
        use_responses_api = True
    else:
        use_responses_api = False

    if use_responses_api:
        openai_messages = await convert_messages_to_openai_responses(messages)
    else:
        openai_messages = await convert_messages_to_openai(messages)

    tools: list[ToolParam] = []
    reasoning: Reasoning | Omit = omit
    include: list[ResponseIncludable] = ["code_interpreter_call.outputs"]

    if model not in ["gpt-5-pro", "gpt-5.2-pro"] and "codex" not in model:
        if code_execution:
            tools.append(
                CodeInterpreter(
                    type="code_interpreter",
                    container=CodeInterpreterContainerCodeInterpreterToolAuto(type="auto")
                )
            )
    tools.append(
        WebSearchToolParam(
            type="web_search",
            search_context_size="high",
        )
    )
    if re.match(r"gpt-5.*", model):
        temperature = 1
    if re.match(r"gpt-5.*", model):
        if thought:
            reasoning = Reasoning(
                effort="high",
                summary="auto",
            )
    if re.match(r"gpt-5.*", model) and "codex" not in model:
        tools.append(
            ImageGeneration(
                type="image_generation",
            )
        )

    text: ResponseTextConfigParam | Omit = omit
    if structured_output_schema:
        text = ResponseTextConfigParam(
            format=ResponseFormatTextJSONSchemaConfigParam(
                name="structured_output",
                schema=structured_output_schema,
                type="json_schema",
                strict=True,
            )
        )

    if use_responses_api:
        if stream:
            return StreamOpenAIResponsesClient(
                model=model,
                messages=openai_messages,
                temperature=temperature,
                api_type=api_type,
                client=client,
                tools=tools,
                reasoning=reasoning,
                include=include,
                text=text,
            )
        else:
            return NonStreamOpenAIResponsesClient(
                model=model,
                messages=openai_messages,
                temperature=temperature,
                api_type=api_type,
                client=client,
                tools=tools,
                reasoning=reasoning,
                include=include,
                text=text,
            )
    else:
        if stream:
            return StreamOpenAIClient(
                model=model,
                messages=openai_messages,
                temperature=temperature,
                api_type=api_type,
                client=client,
            )
        else:
            return NonStreamOpenAIClient(
                model=model,
                messages=openai_messages,
                temperature=temperature,
                api_type=api_type,
                client=client,
            )



from google.genai import types

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.model_message.gemini_message import GeminiMessage, GeminiRole
from llm_bridge.type.message import Message, Role, ContentType, Content


async def convert_message_to_gemini(message: Message) -> GeminiMessage:
    role = message.role

    if role == Role.System:
        role = GeminiRole.User
    if role == Role.Assistant:
        role = GeminiRole.Model

    parts = []

    for content_item in message.contents:
        if content_item.type == ContentType.Text:
            parts.append(types.Part.from_text(text=content_item.data))
        elif content_item.type == ContentType.File:
            file_url = content_item.data
            file_type, sub_type = await get_file_type(file_url)
            if sub_type == "pdf" or file_type in ("image", "video", "audio"):
                file_data, media_type = await media_processor.get_bytes_content_from_url(file_url)
                if media_type == 'video/webm':
                    media_type = 'audio/webm'
                parts.append(types.Part.from_bytes(data=file_data, mime_type=media_type))
            else:
                text_content = types.Part.from_text(
                    text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
                )
                parts.append(text_content)

    return GeminiMessage(parts=parts, role=role)



import json
from importlib.resources import files
from typing import TypedDict

from fastapi import HTTPException


class ModelPrice(TypedDict):
    apiType: str
    model: str
    input: float
    output: float


def load_json_file(package: str, filename: str):
    content = files(package).joinpath(filename).read_text(encoding="utf-8")
    return json.loads(content)


def get_model_prices() -> list[ModelPrice]:
    prices = load_json_file("llm_bridge.resources", "model_prices.json")
    for price in prices:
        price["input"] = float(price["input"])
        price["output"] = float(price["output"])
    return prices


def find_model_prices(api_type: str, model: str) -> ModelPrice | None:
    chat_prices = get_model_prices()
    for chat_price in chat_prices:
        if chat_price['apiType'] == api_type and chat_price['model'] == model:
            return chat_price
    return None


def calculate_chat_cost(
        api_type: str,
        model: str,
        input_tokens: int,
        output_tokens: int
) -> float:
    model_pricing = find_model_prices(api_type, model)

    if model_pricing is None:
        raise HTTPException(status_code=400, detail="Invalid Model")

    input_cost = model_pricing["input"] * (input_tokens / 1_000_000)
    output_cost = model_pricing["output"] * (output_tokens / 1_000_000)
    total_cost = input_cost + output_cost

    return total_cost



from openai.types.responses import ResponseInputTextParam, ResponseInputImageParam, ResponseOutputTextParam, \
    ResponseInputContentParam, EasyInputMessageParam, ResponseOutputMessageParam, ResponseInputFileParam

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type, get_filename_without_timestamp
from llm_bridge.type.message import Message, ContentType, Role
from llm_bridge.type.model_message.openai_responses_message import OpenAIResponsesMessage


async def convert_message_to_openai_responses(message: Message) -> OpenAIResponsesMessage:
    role = message.role
    content: list[ResponseInputContentParam | ResponseOutputTextParam] = []
    contains_pdf = False

    for content_item in message.contents:
        if content_item.type == ContentType.Text:
            if role == Role.Assistant:
                text_content = ResponseOutputTextParam(type="output_text", text=content_item.data, annotations=[])
            else:
                text_content = ResponseInputTextParam(type="input_text", text=content_item.data)
            content.append(text_content)
        elif content_item.type == ContentType.File:
            file_url = content_item.data
            file_type, sub_type = await get_file_type(file_url)
            if file_type == "image":
                image_url = await media_processor.get_openai_image_content_from_url(file_url)
                image_content = ResponseInputImageParam(
                    type="input_image",
                    image_url=image_url,
                    detail="auto"
                )
                content.append(image_content)
            elif sub_type == "pdf":
                contains_pdf = True
                file_data, _ = await media_processor.get_base64_content_from_url(file_url)
                pdf_content = ResponseInputFileParam(
                    type="input_file",
                    filename=get_filename_without_timestamp(file_url),
                    file_data=f"data:application/pdf;base64,{file_data}",
                )
                content.append(pdf_content)
            # TODO: Responses API is currently unsupported for audio input
            # elif file_type == "audio":
            #     encoded_string = await media_processor.get_gpt_audio_content_from_url(file_url)
            #     audio_content = ChatCompletionContentPartInputAudioParam(
            #         type="input_audio",
            #         input_audio=InputAudio(data=encoded_string, format=sub_type)
            #     )
            #     content.append(audio_content)
            else:
                text_content = ResponseInputTextParam(
                    type="input_text",
                    text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
                )
                content.append(text_content)

    # Force system role to user if the message contains a PDF
    if role == Role.System and contains_pdf:
        role = Role.User

    if role in (Role.User, Role.System):
        return EasyInputMessageParam(role=role.value, content=content)
    else:
        return ResponseOutputMessageParam(role=role.value, content=content)



import asyncio
import logging
import os
import sys
from pathlib import Path
from pprint import pprint

from dotenv import load_dotenv

from usage.config import *
from usage.workflow import workflow

script_dir = Path(__file__).parent.resolve()

# Env
load_dotenv(script_dir / ".env")

# Logging Output File
output_path = script_dir / "output.log"
output_path.parent.mkdir(parents=True, exist_ok=True)
output_file = output_path.open("w", encoding="utf-8")
sys.stdout = output_file

logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    stream=output_file
)

api_keys = {
    "OPENAI_API_KEY": os.environ.get("OPENAI_API_KEY"),
    "AZURE_API_KEY": os.environ.get("AZURE_API_KEY"),
    "AZURE_API_BASE": os.environ.get("AZURE_API_BASE"),
    "GITHUB_API_KEY": os.environ.get("GITHUB_API_KEY"),
    "GOOGLE_AI_STUDIO_FREE_TIER_API_KEY": os.environ.get("GOOGLE_AI_STUDIO_FREE_TIER_API_KEY"),
    "GOOGLE_AI_STUDIO_API_KEY": os.environ.get("GOOGLE_AI_STUDIO_API_KEY"),
    "VERTEX_AI_API_KEY": os.environ.get("VERTEX_AI_API_KEY"),
    "ANTHROPIC_API_KEY": os.environ.get("ANTHROPIC_API_KEY"),
    "XAI_API_KEY": os.environ.get("XAI_API_KEY"),
}


async def main():
    input_tokens = 0
    output_tokens = 0
    response = await workflow(
        api_keys,
        messages,
        model,
        api_type,
        temperature,
        stream,
        thought,
        code_execution,
        structured_output_schema,
    )
    text = ""
    thought_text = ""
    code_text = ""
    code_output_text = ""
    files = []

    if stream:
        async for chunk in response:
            pprint(chunk)
            if chunk.text:
                text += chunk.text
            if chunk.thought:
                thought_text += chunk.thought
            if chunk.input_tokens:
                input_tokens = chunk.input_tokens
            if chunk.output_tokens:
                output_tokens += chunk.output_tokens
            if chunk.code:
                code_text += chunk.code
            if chunk.code_output:
                code_output_text += chunk.code_output
            if chunk.files:
                files.extend(chunk.files)
    else:
        pprint(response)
        text = response.text
        thought_text = response.thought
        code_text = response.code
        code_output_text = response.code_output
        input_tokens = response.input_tokens
        output_tokens = response.output_tokens
        files = response.files
    total_cost = calculate_chat_cost(api_type, model, input_tokens, output_tokens)
    print(f"Thought:\n{thought_text}\n")
    print(f"Code:\n{code_text}\n")
    print(f"Code Output:\n{code_output_text}\n")
    print(f"Text:\n{text}\n")
    print(f"Files:\n{files}\n")
    print(f'Input tokens: {input_tokens}, Output tokens: {output_tokens}, Total cost: ${total_cost}')


if __name__ == "__main__":
    asyncio.run(main())
    output_file.close()



from typing import AsyncGenerator, Any

from llm_bridge import *


async def workflow(
        api_keys: dict[str, str],
        messages: list[Message],
        model: str,
        api_type: str,
        temperature: float,
        stream: bool,
        thought: bool,
        code_execution: bool,
        structured_output_schema: dict[str, Any] | None,
) -> ChatResponse | AsyncGenerator[ChatResponse, None]:
    await preprocess_messages(messages, api_type)

    chat_client = await create_chat_client(
        api_keys=api_keys,
        messages=messages,
        model=model,
        api_type=api_type,
        temperature=temperature,
        stream=stream,
        thought=thought,
        code_execution=code_execution,
        structured_output_schema=structured_output_schema,
    )

    if stream:
        return chat_client.generate_stream_response()
    else:
        return await chat_client.generate_non_stream_response()



from anthropic.types import TextBlockParam, ImageBlockParam, DocumentBlockParam, Base64ImageSourceParam, \
    Base64PDFSourceParam

from llm_bridge.logic.chat_generate import media_processor
from llm_bridge.logic.message_preprocess.file_type_checker import get_file_type
from llm_bridge.type.message import Message, Role, ContentType
from llm_bridge.type.model_message.claude_message import ClaudeMessage, ClaudeRole


async def convert_message_to_claude(message: Message) -> ClaudeMessage:
    role = message.role
    
    if role in (Role.System, Role.User):
        role = ClaudeRole.User
    if role == Role.Assistant:
        role = ClaudeRole.Assistant

    claude_content: list[TextBlockParam | ImageBlockParam | DocumentBlockParam] = []

    for content_item in message.contents:
        if content_item.type == ContentType.Text:
            text_content = TextBlockParam(type="text", text=content_item.data)
            claude_content.append(text_content)
        elif content_item.type == ContentType.File:
            file_url = content_item.data
            file_type, sub_type = await get_file_type(file_url)
            if file_type == "image":
                base64_image, media_type = await media_processor.get_base64_content_from_url(file_url)
                image_content = ImageBlockParam(
                    type="image",
                    source=Base64ImageSourceParam(
                        type="base64",
                        media_type=media_type,
                        data=base64_image
                    )
                )
                claude_content.append(image_content)
            elif sub_type == "pdf":
                file_data, media_type = await media_processor.get_base64_content_from_url(file_url)
                pdf_content = DocumentBlockParam(
                    type="document",
                    source=Base64PDFSourceParam(
                        type="base64",
                        media_type=media_type,
                        data=file_data
                    ),
                )
                claude_content.append(pdf_content)
            else:
                text_content = TextBlockParam(
                    type="text",
                    text=f"\n{file_url}: {file_type}/{sub_type} not supported by the current model.\n"
                )
                claude_content.append(text_content)

    return ClaudeMessage(role=role, content=claude_content)
